#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004, 2017
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : TLB.tcl
##
## Purpose    : Mambo TLB display and triggering functions
##
#####################################################
#####################################################

namespace eval TLB {
    variable version            "1.0"
    variable debug              0
    variable silent             0
}

package provide TLB $TLB::version

####
# Display an TLB
#
proc TLB::display {{tlb .*} {mc 0:0} {sim mysim}} {
    if {![regexp {((\d):)?(\d)} $mc match x m c]} {
        puts "TLB::display -- malformed mcm:cpu id '$mc'"
        return -code error
    }

    if {[catch {array set tlb_info [$sim mcm $m cpu $c tlb display $tlb]} errs]} {
        puts "TLB::display -- cannot issue tlb display command ($errs)"
        return -code error
    }

    foreach e [array names tlb_info] {
        puts "TLB: $e"
        array unset row
        array set row $tlb_info($e)
        foreach r [array names row] {
            puts "\tROW: $r"
            array unset col
            array set col $row($r)
            foreach c [array names col] {
                puts "\t\tCOL: $c\t$col($c)"
            }
        }
    }
}

####
# TLB Trigger Add
#
proc TLB::trigger_add {args} {
    puts "TLB::trigger_add $args"

    array set p $args
    TLB::display $p(tlb) $p(mcm):$p(cpu) $p(sim)
}

####
# TLB Trigger central dispatch
#
proc TLB::trigger_dispatch {args} {
    array set p $args
    switch -exact -- $p(op) {
        add {
            eval TLB::trigger_add $args
        }
        default {
           puts "*** TLB: $args"
       }
   }
}

####
# TLB trigger enable
#
proc TLB::trigger_enable {{tlb .*} {mc 0:0} {sim mysim}} {
    if {![regexp {((\d):)?(\d)} $mc match x m c]} {
        puts "TLB::trigger_enable -- malformed mcm:cpu id '$mc'"
        return -code error
    }

    if {[catch {$sim mcm $m cpu $c trigger tlb set $tlb [list TLB::trigger_dispatch]} errs]} {
        puts "TLB::trigger_enable -- cannot issue tlb trigger command ($errs)"
        return -code error
    }
}
