#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2016, 2017
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  p9-devtree-skiboot.tcl
#
#
#------------------------------------------------------------------------
#  Add the needed pieces to the device tree to allow Linux on top of
#  skiboot to boot
# 

set num_cores [ mysim mcm 0 display number_of_cpus ]
set num_threads_per_core [ expr [ mysim mcm 0 display number_of_threads ] / $num_cores ]

# instantiate xscom
set xscom_base 0x1A0000000000
mysim xscom create $xscom_base

set root_node [mysim of find_device "/"]

mysim of addprop $root_node string "epapr-version" "ePAPR-1.0"
mysim of setprop $root_node "compatible" "ibm,powernv"

set mem0_node [mysim of find_device "/memory@0"]
mysim of addprop $mem0_node int "ibm,chip-id" 0

set xscom_node [ mysim of addchild $root_node xscom [format %x $xscom_base]]
set reg [list $xscom_base 0x10000000]
mysim of addprop $xscom_node array64 "reg" reg
mysim of addprop $xscom_node empty "scom-controller" ""
mysim of addprop $xscom_node int "ibm,chip-id" 0
mysim of addprop $xscom_node int "#address-cells" 1
mysim of addprop $xscom_node int "#size-cells" 1
set compat [list]
lappend compat "ibm,xscom"
lappend compat "ibm,power8-xscom"
set compat [of::encode_compat $compat]
mysim of addprop $xscom_node byte_array "compatible" $compat

# setup boot arguments for bogus disk
#mysim of setprop [mysim of find_device /chosen] bootargs "rw root=/dev/mambobd0 console=hvc0 "
of::set_bootargs "root=/dev/mambobd0 rw console=hvc0"

# ===== Setup the "ibm,processor-segment-sizes" property =====
#  -> add all of the supported segment sizes

set seg_sizes [ list ]
# 28 bit segments and 40 bit segments
lappend seg_sizes 0x0000001c00000028
# unimplemented
lappend seg_sizes 0xffffffffffffffff

# ===== Setup the "ibm,processor-page-sizes" property =====
#  -> add all of the supported page sizes

set pg_sizes [ list ]
# 4K page size
lappend pg_sizes 0x0c
# 64K page size
lappend pg_sizes 0x10
# 16M page size
lappend pg_sizes 0x18
# 16G page size
lappend pg_sizes 0x22

# ===== Setup the "ibm,segment-page-sizes" property =====
#  -> add all of the supported page sizes

set seg_pgsizes [ list ]
# base page size for 4K
lappend seg_pgsizes 0xc
# SLB encoding (slbenc)
lappend seg_pgsizes 0x0
# number of LPTE encodings (lpnum)
lappend seg_pgsizes 0x3
# LPTE1 for 4K (shift, penc)
lappend seg_pgsizes 0xc 0x0
# LPTE2 for 64K
lappend seg_pgsizes 0x10 0x07
# LPTE3 for 16MB
lappend seg_pgsizes 0x18 0x38

# base page size for 64K
lappend seg_pgsizes 0x10
# SLB encoding (slbenc)
lappend seg_pgsizes 0x110
# number of LPTE encodings (lpnum)
lappend seg_pgsizes 0x2
# LPTE1 for 64K (shift, penc)
lappend seg_pgsizes 0x10 0x01
# LPTE2 for 16M
lappend seg_pgsizes 0x18 0x08

# base page size for 16M
lappend seg_pgsizes 0x18
# SLB encoding (slbenc)
lappend seg_pgsizes 0x100
# number of LPTE encodings (lpnum)
lappend seg_pgsizes 0x1
# LPTE1 for 16M (shift, penc)
lappend seg_pgsizes 0x18 0x0

# base page size for 16G
lappend seg_pgsizes 0x22
# SLB encoding (slbenc)
lappend seg_pgsizes 0x120
# number of LPTE encodings (lpnum)
lappend seg_pgsizes 0x1
# LPTE1 for 16G (shift, penc)
lappend seg_pgsizes 0x22 0x03

# ===== Setup the "ibm,processor-radix-AP-encodings" property =====
#  -> add all of the radix page sizes as their AP encodings

set pgsizes_radix {}
# 4K pages
lappend pgsizes_radix 0x0000000c
# 64K pages
lappend pgsizes_radix 0xa0000010
# 2M pages
lappend pgsizes_radix 0x20000015
# 1G pages
lappend pgsizes_radix 0x4000001e


set cpus_node [mysim of find_device "/cpus"]
set cpu [mysim of child $cpus_node]

# iterate through the CPU nodes and update them
set curr_pir 0
set curr_core 0
while { $cpu != 0 } {
    mysim of addprop $cpu array64 "ibm,processor-segment-sizes" seg_sizes
    mysim of addprop $cpu array "ibm,processor-page-sizes" pg_sizes
    mysim of addprop $cpu array "ibm,segment-page-sizes" seg_pgsizes
    mysim of addprop $cpu array "ibm,processor-radix-AP-encodings" pgsizes_radix
    mysim of addprop $cpu int "ibm,chip-id" $curr_core
    mysim of addprop $cpu int "ibm,pir" $curr_pir
    mysim of setprop $cpu "i-cache-size" 32768
    mysim of setprop $cpu "i-cache-sets" 32
    mysim of setprop $cpu "d-cache-size" 32768
    mysim of setprop $cpu "d-cache-sets" 32
    incr curr_core
    incr curr_pir $num_threads_per_core
    set cpu [mysim of peer $cpu]
}
